"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const os_1 = require("os");
const path = require("path");
const helpers = require("../common/helpers");
const nativescript_doctor_1 = require("nativescript-doctor");
class DoctorService {
    constructor($analyticsService, $hostInfo, $logger, $childProcess, $opener, $prompter, $versionsService) {
        this.$analyticsService = $analyticsService;
        this.$hostInfo = $hostInfo;
        this.$logger = $logger;
        this.$childProcess = $childProcess;
        this.$opener = $opener;
        this.$prompter = $prompter;
        this.$versionsService = $versionsService;
    }
    printWarnings(configOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            const warnings = yield nativescript_doctor_1.doctor.getWarnings();
            const hasWarnings = warnings.length > 0;
            const hasAndroidWarnings = warnings.filter(warning => _.includes(warning.platforms, nativescript_doctor_1.constants.ANDROID_PLATFORM_NAME)).length > 0;
            if (hasAndroidWarnings) {
                this.printPackageManagerTip();
            }
            if (!configOptions || configOptions.trackResult) {
                yield this.$analyticsService.track("DoctorEnvironmentSetup", hasWarnings ? "incorrect" : "correct");
            }
            if (hasWarnings) {
                warnings.map(warning => {
                    this.$logger.warn(warning.warning);
                    this.$logger.out(warning.additionalInformation);
                });
                this.$logger.info("There seem to be issues with your configuration.");
                if (this.$hostInfo.isDarwin) {
                    yield this.promptForHelp(DoctorService.DarwinSetupDocsLink, DoctorService.DarwinSetupScriptLocation, []);
                }
                else if (this.$hostInfo.isWindows) {
                    yield this.promptForHelp(DoctorService.WindowsSetupDocsLink, DoctorService.WindowsSetupScriptExecutable, DoctorService.WindowsSetupScriptArguments);
                }
                else {
                    yield this.promptForDocs(DoctorService.LinuxSetupDocsLink);
                }
            }
            try {
                yield this.$versionsService.checkComponentsForUpdate();
            }
            catch (err) {
                this.$logger.error("Cannot get the latest versions information from npm. Please try again later.");
            }
            return hasWarnings;
        });
    }
    promptForDocs(link) {
        return __awaiter(this, void 0, void 0, function* () {
            if (yield this.$prompter.confirm("Do you want to visit the official documentation?", () => helpers.isInteractive())) {
                this.$opener.open(link);
            }
        });
    }
    promptForHelp(link, commandName, commandArguments) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.promptForDocs(link);
            if (yield this.$prompter.confirm("Do you want to run the setup script?", () => helpers.isInteractive())) {
                yield this.$childProcess.spawnFromEvent(commandName, commandArguments, "close", { stdio: "inherit" });
            }
        });
    }
    printPackageManagerTip() {
        if (this.$hostInfo.isWindows) {
            this.$logger.out("TIP: To avoid setting up the necessary environment variables, you can use the chocolatey package manager to install the Android SDK and its dependencies." + os_1.EOL);
        }
        else if (this.$hostInfo.isDarwin) {
            this.$logger.out("TIP: To avoid setting up the necessary environment variables, you can use the Homebrew package manager to install the Android SDK and its dependencies." + os_1.EOL);
        }
    }
}
DoctorService.DarwinSetupScriptLocation = path.join(__dirname, "..", "..", "setup", "mac-startup-shell-script.sh");
DoctorService.DarwinSetupDocsLink = "https://docs.nativescript.org/start/ns-setup-os-x";
DoctorService.WindowsSetupScriptExecutable = "powershell.exe";
DoctorService.WindowsSetupScriptArguments = ["start-process", "-FilePath", "PowerShell.exe", "-NoNewWindow", "-Wait", "-ArgumentList", '"-NoProfile -ExecutionPolicy Bypass -Command iex ((new-object net.webclient).DownloadString(\'https://www.nativescript.org/setup/win\'))"'];
DoctorService.WindowsSetupDocsLink = "https://docs.nativescript.org/start/ns-setup-win";
DoctorService.LinuxSetupDocsLink = "https://docs.nativescript.org/start/ns-setup-linux";
$injector.register("doctorService", DoctorService);
